@@ -1,7 +1,7 @@
 /* -*- buffer-read-only: t -*- vi: set ro:
  *
  * Prototypes for autoopts
- * Generated Sun Jul 25 09:16:13 PDT 2010
+ * Generated Tue Nov 16 17:41:54 PST 2010
  */
 #ifndef AUTOOPTS_PROTO_H_GUARD
 #define AUTOOPTS_PROTO_H_GUARD 1
@@ -84,6 +84,9 @@ loadOptionLine(
 /*
  *  Extracted from nested.c
  */
+LOCAL void
+unload_arg_list(tArgList* pAL);
+
 LOCAL tOptionValue*
 optionLoadNested(char const* pzTxt, char const* pzName, size_t nameLen);
 
